# coding: utf-8
import os
from pprint import pprint
from wechatarticles.ReadOutfile import Reader
from wechatarticles import ArticlesAPI
from wechatarticles import tools

if __name__ == '__main__':

    official_cookie = "official_cookie"
    token = "token"
    appmsg_token = "appmsg_token"
    wechat_cookie = "wechat_cookie"

    nickname = "nickname"

    # 手动输入所有参数
    test = ArticlesAPI(official_cookie=official_cookie,
                       token=token,
                       appmsg_token=appmsg_token,
                       wechat_cookie=wechat_cookie)


    # 自定义爬取，每次爬取5篇以上
    data = test.complete_info(nickname=nickname, begin="0")
    print(len(data))
    pprint(data)

    # 自定义从某部分开始爬取，持续爬取，直至爬取失败为止，一次性最多爬取40篇（功能未测试，欢迎尝试）
    datas = test.continue_info(nickname=nickname, begin="0")

    tools.save_json("test.json", data)


# coding: utf-8
"""
辅助脚本函数
"""
import html
import json
import os
import re
import time

import requests
from bs4 import BeautifulSoup as bs

from .GetUrls import PCUrls

base_columns = ['url', 'title', 'date', 'headlines', 'copyright']
A_columns = ['read_num', 'old_like_num', 'like_num']
B_columns = ['comments_num', 'comments_content', 'comments_like_num']
C_columns = ['content', 'content_num', 'pic_num']
mode_columns = {
    1: A_columns,
    2: B_columns,
    3: C_columns,
    4: A_columns + B_columns,
    5: A_columns + C_columns,
    6: B_columns + C_columns,
    7: A_columns + B_columns + C_columns
}

ctext = '你的访问过于频繁，需要从微信打开验证身份，是否需要继续访问当前页面'


# url, readnum likenum
def flatten(x):
    return [y for l in x for y in flatten(l)] if type(x) is list else [x]


def remove_duplicate_json(fname):
    # 删除json中重复的数据
    # fname: xxx.json
    with open(fname, 'r', encoding='utf-8') as f:
        data = f.readlines()

    id_re = re.compile(r'datetime": (.+), "fakeid"')
    sort_func = lambda line: id_re.findall(line)[0]

    list_data = list(set(data))
    sort_data = sorted(list_data, key=sort_func)[::-1]

    # sort_data = sorted(list(set_data),
    #                    key=lambda line: re.findall(
    #                        r'datetime": (.+), "fakeid"', line)[0])[::-1]

    with open(fname, 'w', encoding='utf-8') as f:
        f.writelines(sort_data)


def end_func(timestamp, end_timestamp):
    if timestamp < end_timestamp:
        print(timestamp, end_timestamp)
        return True
    return False


def transfer_url(url):
    url = html.unescape(html.unescape(url))
    return eval(repr(url).replace('\\', ''))


def save_f(fname):
    i = 1
    while True:
        if os.path.isfile('{}.json'.format(fname)):
            i += 1
            fname += '-' + str(i)
        else:
            break

    return fname


# verify_lst = ["mp.weixin.qq.com", "__biz", "mid", "sn", "idx"]
verify_lst = ["mp.weixin.qq.com", "__biz", "mid", "idx"]


def verify_url(article_url):
    for string in verify_lst:
        if string not in article_url:
            return False
    return True


def get_content(url, cookie):
    headers = {
        'user-agent':
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.125 Safari/537.36',
        'cookie': cookie
    }
    html_text = requests.get(url.strip(), headers=headers).text

    soup = bs(html_text, 'lxml')
    if ctext in html_text:
        assert 1 == 2
    # js加载
    # html.text.split('var content = ')[1].split('var')[0].strip()
    # soup.find(id="js_panel_like_title").text
    try:
        body = soup.find(class_="rich_media_area_primary_inner")
        content_p = body.find(class_="rich_media_content")
        if content_p:
            imgs = body.find_all('img')
            return content_p.text.strip(), len(
                content_p.text.strip()), len(imgs)
        else:
            content_p = soup.find(id="js_panel_like_title").text.strip()
            return content_p, len(content_p), 0
        # with open(txt_name, 'w', encoding='utf-8') as f:
        # f.write(content_p.text)
    except:
        return '', 0, 0


def copyright_num(copyright_stat):
    if copyright_stat == 11:
        return 1  # 标记原创
    else:
        return 0


def copyright_num_detailed(copyright_stat):
    copyright_stat_lst = [14, 12, 201]
    if copyright_stat == 11:
        return 1  # 标记原创
    elif copyright_stat == 100:
        return 0  # 荐号
    elif copyright_stat == 101:
        return 2  # 转发
    elif copyright_stat == 0:
        return 3  # 来源非微信文章
    elif copyright_stat == 1:
        return 4  # 形容词（xxx的公众号）
    elif copyright_stat in copyright_stat_lst:
        return 5
    else:
        return None


def read_nickname(fname):
    # 读取数据
    with open(fname, 'r', encoding='utf-8') as f:
        haved_data = f.readlines()
    return [line.split(', ') for line in haved_data]


def get_history_urls(biz,
                     uin,
                     key,
                     lst=[],
                     start_timestamp=0,
                     count=10,
                     endcount=99999):
    t = PCUrls(biz=biz, uin=uin, cookie='')
    try:
        while True:
            res = t.get_urls(key, offset=count)
            if res == []:
                break
            count += 10
            print(count)
            lst.append(res)
            dt = res[-1]["comm_msg_info"]["datetime"]
            if dt <= start_timestamp or count >= endcount:
                break
            time.sleep(5)
    except KeyboardInterrupt as e:
        print('程序手动中断')
        return lst
    except Exception as e:
        print(e)
        print("获取文章链接失败。。。退出程序")
        assert 1 == 2
    finally:
        return lst



# coding: utf-8
from selenium import webdriver
import re
import time
from functools import reduce
from pprint import pprint
import pickle
"""
note: 需要使用selenium，chrome版本需要与chromedriver版本对应。具体见https://chromedriver.storage.googleapis.com/
"""


def login(username, password):
    #打开微信公众号登录页面
    driver.get('https://mp.weixin.qq.com/')
    driver.maximize_window()
    time.sleep(3)
    # 自动填充帐号密码
    driver.find_element_by_xpath(
        "//*[@id=\"header\"]/div[2]/div/div/form/div[1]/div[1]/div/span/input"
    ).clear()
    driver.find_element_by_xpath(
        "//*[@id=\"header\"]/div[2]/div/div/form/div[1]/div[1]/div/span/input"
    ).send_keys(username)
    driver.find_element_by_xpath(
        "//*[@id=\"header\"]/div[2]/div/div/form/div[1]/div[2]/div/span/input"
    ).clear()
    driver.find_element_by_xpath(
        "//*[@id=\"header\"]/div[2]/div/div/form/div[1]/div[2]/div/span/input"
    ).send_keys(password)

    time.sleep(1)
    #自动点击登录按钮进行登录
    driver.find_element_by_xpath(
        "//*[@id=\"header\"]/div[2]/div/div/form/div[4]/a").click()
    # 拿手机扫二维码！
    time.sleep(15)


def open_link(nickname):
    # 进入新建图文素材
    driver.find_element_by_xpath(
        '//*[@id="menuBar"]/li[4]/ul/li[3]/a/span/span').click()
    driver.find_element_by_xpath(
        '//*[@id="js_main"]/div[3]/div[1]/div[2]/div[2]/div/a[1]').click()
    time.sleep(10)

    # 切换到新窗口
    for handle in driver.window_handles:
        if handle != driver.current_window_handle:
            driver.switch_to_window(handle)

    # 点击超链接
    driver.find_element_by_xpath('//*[@id="edui23_body"]/div').click()
    time.sleep(3)
    # 点击查找文章
    driver.find_element_by_xpath(
        '//*[@id="myform"]/div[3]/div[1]/div/label[2]').click()
    # 输入公众号名称
    driver.find_element_by_xpath(
        '//*[@id="myform"]/div[3]/div[3]/div[1]/div/span[1]/input').clear()
    driver.find_element_by_xpath(
        '//*[@id="myform"]/div[3]/div[3]/div[1]/div/span[1]/input').send_keys(
            nickname)
    # 点击搜索
    driver.find_element_by_xpath(
        '//*[@id="myform"]/div[3]/div[3]/div[1]/div/span[1]/a[2]').click()
    time.sleep(3)
    # 点击第一个公众号
    driver.find_element_by_xpath(
        '//*[@id="myform"]/div[3]/div[3]/div[2]/div/div[1]/div/div[1]/div[3]/p[2]'
    ).click()
    time.sleep(3)


def get_url_title(html):
    lst = []
    for item in driver.find_elements_by_class_name('my_link_item'):
        temp_dict = {
            'date': item.text.split('\n')[0],
            'url': item.find_element_by_tag_name('a').get_attribute('href'),
            'title': item.text.split('\n')[1],
        }
        lst.append(temp_dict)
    return lst


#用webdriver启动谷歌浏览器
driver = webdriver.Chrome(executable_path=chromedriver_path)

nickname = ''  # 公众号名称
username = ''  # 账号
password = ''  # 密码
login(username, password)
open_link(nickname)
page_num = int(
    driver.find_elements_by_class_name('page_num')[-1].text.split('/')
    [-1].lstrip())

# 点击下一页
url_title_lst = get_url_title(driver.page_source)

for _ in range(1, page_num):
    try:
        pagination = driver.find_elements_by_class_name('pagination')[1]
        pagination.find_elements_by_tag_name('a')[2].click()
        time.sleep(5)
        url_title_lst += get_url_title(driver.page_source)
    except:
        # 保存
        with open('data.pickle', 'wb') as f:
            pickle.dump(url_title_lst, f)
        print("第{}页失败".format(_))
        break

with open('data2.pickle', 'wb') as f:
    pickle.dump(data, f)
# 读取
with open('data.pickle', 'rb') as f:
    b = pickle.load(f)



# coding:  utf-8
import hashlib
import os

import requests
from requests.cookies import cookielib


class ArticlesUrls(object):
    """
    获取需要爬取的微信公众号的推文链接
    """
    def __init__(self, cookie, token, proxies={'http': None, 'https': None}):
        """
        初始化参数
        Parameters
        ----------
        token : str
            登录微信公众号平台之后获取的token
        cookie : str
            登录微信公众号平台之后获取的cookie

        Returns
        -------
            None
        """
        self.s = requests.session()
        self.headers = {
            "User-Agent":
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.62 Safari/537.36"
        }
        self.params = {
            "lang": "zh_CN",
            "f": "json",
        }

        # 手动输入cookie和token登录
        self.__verify_str(cookie, "cookie")
        self.__verify_str(token, "token")
        self.headers["Cookie"] = cookie
        self.params["token"] = token
        self.proxies = proxies

    def __verify_str(self, input_string, param_name):
        """
        验证输入是否为字符串
        Parameters
        ----------
        input_string: str
            输入
        param_name: str
            需要验证的参数名
        Returns
        ----------
            None
        """
        if not isinstance(input_string, str):
            raise TypeError("{} must be an instance of str".format(param_name))

    def __save_login_qrcode(self, img):
        """
        存储和显示登录二维码
        Parameters
        ----------
        img: str
            获取到的二维码数据

        Returns
        -------
            None
        """
        import matplotlib.pyplot as plt
        from PIL import Image
        # 存储二维码
        with open("login.png", "wb+") as fp:
            fp.write(img.content)
        # 显示二维码， 这里使用plt的原因是： 等待用户扫描完之后手动关闭窗口继续运行；否则会直接运行
        try:
            img = Image.open("login.png")
        except Exception:
            raise TypeError(u"账号密码输入错误，请重新输入")
        plt.figure()
        plt.imshow(img)
        plt.show()

    def __save_cookie(self, username):
        """
        存储cookies, username用于文件命名
        Parameters
        ----------
        username: str
            用户账号

        Returns
        -------
            None
        """
        #实例化一个LWPcookiejar对象
        new_cookie_jar = cookielib.LWPCookieJar(username + '.txt')

        #将转换成字典格式的RequestsCookieJar（这里我用字典推导手动转的）保存到LWPcookiejar中
        requests.utils.cookiejar_from_dict(
            {c.name: c.value
             for c in self.s.cookies}, new_cookie_jar)

        #保存到本地文件
        new_cookie_jar.save('cookies/' + username + '.txt',
                            ignore_discard=True,
                            ignore_expires=True)

    def __read_cookie(self, username):
        """
        读取cookies, username用于文件命名
        Parameters
        ----------
        username: str
            用户账号

        Returns
        -------
            None
        """
        #实例化一个LWPCookieJar对象
        load_cookiejar = cookielib.LWPCookieJar()
        #从文件中加载cookies(LWP格式)
        load_cookiejar.load('cookies/' + username + '.txt',
                            ignore_discard=True,
                            ignore_expires=True)
        #工具方法转换成字典
        load_cookies = requests.utils.dict_from_cookiejar(load_cookiejar)
        #工具方法将字典转换成RequestsCookieJar，赋值给session的cookies.
        self.s.cookies = requests.utils.cookiejar_from_dict(load_cookies)

    def __md5_passwd(self, password):
        """
        微信公众号的登录密码需要用md5方式进行加密
        Parameters
        ----------
        password: str
            加密前的字符串

        Returns
        -------
        str：
            加密后的字符串
        """
        m5 = hashlib.md5()
        m5.update(password.encode('utf-8'))
        pwd = m5.hexdigest()
        return pwd

    def __startlogin_official(self, username, password):
        """
        开始登录微信公众号平台，获取Cookies
        Parameters
        ----------
        username: str
            用户账号
        password: str
            用户密码
        Returns
        -------
            None
        """
        # 进行md5加密，一些post的参数
        pwd = self.__md5_passwd(password)
        data = {
            "username": username,
            "userlang": "zh_CN",
            "token": "",
            "pwd": pwd,
            "lang": "zh_CN",
            "imgcode": "",
            "f": "json",
            "ajax": "1"
        }

        # 增加headers的keys
        self.headers["Host"] = "mp.weixin.qq.com"
        self.headers["Origin"] = "https://mp.weixin.qq.com"
        self.headers["Referer"] = "https://mp.weixin.qq.com/"

        # 账号密码post的url
        bizlogin_url = "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=startlogin"
        # 获取二维码的url
        qrcode_url = "https://mp.weixin.qq.com/cgi-bin/loginqrcode?action=getqrcode&param=4300&rd=928"

        # 账号密码登录，获取二维码，等待用户扫描二维码，需手动关闭二维码窗口
        self.s.post(bizlogin_url, headers=self.headers, data=data)
        img = self.s.get(qrcode_url)
        self.__save_login_qrcode(img)

        # 去除之后不用的headers的key
        self.headers.pop("Host")
        self.headers.pop("Origin")
        # 获取token
        self.__login_official(username, password)

    def __login_official(self, username, password):
        """
        正式登录微信公众号平台，获取token
        Parameters
        ----------
        username: str
            用户账号
        password: str
            用户密码
        Returns
        -------
            None
        """
        # 设定headers的referer的请求
        referer = "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=validate&lang=zh_CN&account={}".format(
            username)
        self.headers["Referer"] = referer

        # 获取token的data
        data = {
            "userlang": "zh_CN",
            "token": "",
            "lang": "zh_CN",
            "f": "json",
            "ajax": "1",
        }
        # 获取token的url
        bizlogin_url = "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=login"
        res = self.s.post(bizlogin_url,
                          data=data,
                          headers=self.headers,
                          proxies=self.proxies).json()

        try:
            # 截取字符串中的token参数
            token = res["redirect_url"].split("=")[-1]
            self.params["token"] = token
            # self.__save_cookie(username)
            self.headers.pop("Referer")
        except Exception:
            # 获取token失败，重新扫码登录
            print("please try again")
            self.__startlogin_official(username, password)

    def official_info(self, nickname, begin=0, count=5):
        """
        根据关键词返回相关公众号的信息
        Parameters
        ----------
        nickname : str
            需要爬取公众号名称
        begin: str or int
            起始爬取的页数
        count: str or int
            每次爬取的数量，1-5

        Returns
        -------
        list:
            相关公众号的对应信息
            [
                {
                'alias': 公众号别名,
                'fakeid': 公众号唯一id,
                'nickname': 公众号名称,
                'round_head_img': 公众号头像的url,
                'service_type': 1公众号性质
                },
            ...
            ]
        """
        self.__verify_str(nickname, "nickname")
        # 搜索公众号的url
        search_url = "https://mp.weixin.qq.com/cgi-bin/searchbiz"

        # 增加/更改请求参数
        params = {
            "query": nickname,
            "count": str(count),
            "action": "search_biz",
            "ajax": "1",
            "begin": str(begin)
        }
        self.params.update(params)

        try:
            # 返回与输入公众号名称最接近的公众号信息
            official = self.s.get(search_url,
                                  headers=self.headers,
                                  params=self.params,
                                  proxies=self.proxies)
            return official.json()["list"]
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

    def articles_nums(self, nickname):
        """
        获取公众号的总共发布的文章数量
        Parameters
        ----------
        nickname : str
            需要爬取公众号名称

        Returns
        -------
        int
            文章总数
        """
        self.__verify_str(nickname, "nickname")
        try:
            return self.__get_articles_data(nickname, begin="0")["app_msg_cnt"]
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

    def articles(self, nickname, begin=0, count=5):
        """
        获取公众号的每页的文章信息
        Parameters
        ----------
        nickname : str
            需要爬取公众号名称
        begin: str or int
            起始爬取的页数
        count: str or int
            每次爬取的数量，1-5

        Returns
        -------
        list:
            由每个文章信息构成的数组
            [
              {
                'aid': '2650949647_1',
                'appmsgid': 2650949647,
                'cover': 封面的url'digest': 文章摘要,
                'itemidx': 1,
                'link': 文章的url,
                'title': 文章标题,
                'update_time': 更新文章的时间戳
              },
            ]
        如果list为空则说明没有相关文章
        """
        self.__verify_str(nickname, "nickname")
        try:
            return self.__get_articles_data(nickname,
                                            begin=str(begin),
                                            count=str(count))["app_msg_list"]
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

    def lastest_articles(self, biz):
        """
        获取公众号的每页的文章信息
        Parameters
        ----------
        biz : str
            公众号的biz

        Returns
        -------
        list:
            由每个文章信息构成的数组
            [
              {
                'aid': '2650949647_1',
                'appmsgid': 2650949647,
                'cover': 封面的url'digest': 文章摘要,
                'itemidx': 1,
                'link': 文章的url,
                'title': 文章标题,
                'update_time': 更新文章的时间戳
              },
            ]
        如果list为空则说明没有相关文章
        """
        try:
            return self.__get_articles_data("", begin="0",
                                            biz=biz)["app_msg_list"]
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

    def __get_articles_data(self,
                            nickname,
                            begin,
                            biz=None,
                            count=5,
                            type_="9",
                            action="list_ex",
                            query=None):
        """
        获取公众号文章的一些信息
        Parameters
        ----------
        nickname : str
            需要爬取公众号名称
        begin: str or int
            起始爬取的页数
        biz : str
            公众号的biz
        count: str or int
            每次爬取的数量，1-5
        type_: str or int
            获取数据的方式，暂不知道具体用途
        action: str
            请求之后的行为动作，"list_ex"获取文章信息的json
        Returns
        -------
        json:
            文章信息的json
            {
              'app_msg_cnt': 公众号发文章总数,
              'app_msg_list': 　一个数组(参看get_articles函数),
              'base_resp': {
                'err_msg': 'ok',
                'ret': 0
              }
            }
        """
        # 获取文章信息的url
        appmsg_url = "https://mp.weixin.qq.com/cgi-bin/appmsg"

        try:
            if nickname != "":
                # 获取公众号的fakeid
                official_info = self.official_info(nickname)
                self.params["fakeid"] = official_info[0]["fakeid"]
            elif biz != None:
                self.params["fakeid"] = biz
            else:
                raise Exception(u"请输入biz或者nickname")
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

        # 增加/更改请求参数
        params = {
            "query": query if query != None else "",
            "begin": str(begin),
            "count": str(count),
            "type": str(type_),
            "action": action
        }
        self.params.update(params)

        data = self.s.get(appmsg_url,
                          headers=self.headers,
                          params=self.params,
                          proxies=self.proxies)
        return data.json()



# coding:  utf-8
import re

import requests


class ArticlesInfo(object):
    """
    登录WeChat，获取更加详细的推文信息。如点赞数、阅读数、评论等
    """
    def __init__(self,
                 appmsg_token,
                 cookie,
                 proxies={
                     'http': None,
                     'https': None
                 }):
        """
        初始化参数
        Parameters
        ----------
        cookie: str
            点开微信公众号文章抓包工具获取的cookie
        appmsg_token: str
            点开微信公众号文章抓包工具获取的appmsg_token

        Returns
        -------
        None
        """
        self.s = requests.session()
        self.appmsg_token = appmsg_token
        self.headers = {
            "User-Agent":
            "Mozilla/5.0 AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0Chrome/57.0.2987.132 MQQBrowser/6.2 Mobile",
            "Cookie": cookie
        }
        self.data = {
            "is_only_read": "1",
            "is_temp_url": "0",
            "appmsg_type": "9",  # 新参数，不加入无法获取like_num
        }
        self.proxies = proxies

    def __verify_url(self, article_url):
        """
        简单验证文章url是否符合要求
        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        None
        """
        verify_lst = ["mp.weixin.qq.com", "__biz", "mid", "sn", "idx"]
        for string in verify_lst:
            if string not in article_url:
                raise Exception(
                    "params is error, please check your article_url")

    def read_like_nums(self, article_url):
        """
        获取阅读数和点赞数
        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        (int, int):
            阅读数、点赞数
        """
        try:
            appmsgstat = self.__get_appmsgext(article_url)["appmsgstat"]
            return appmsgstat["read_num"], appmsgstat["like_num"], appmsgstat[
                "old_like_num"]
        except Exception:
            raise Exception("params is error, please check your article_url")

    def comments(self, article_url):
        """
        获取文章评论
        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        json:
            {
                "base_resp": {
                    "errmsg": "ok",
                    "ret": 0
                },
                "elected_comment": [
                    {
                        "content": 用户评论文字,
                        "content_id": "6846263421277569047",
                        "create_time": 1520098511,
                        "id": 3,
                        "is_from_friend": 0,
                        "is_from_me": 0,
                        "is_top": 0, 是否被置顶
                        "like_id": 10001,
                        "like_num": 3,
                        "like_status": 0,
                        "logo_url": "http://wx.qlogo.cn/mmhead/OibRNdtlJdkFLMHYLMR92Lvq0PicDpJpbnaicP3Z6kVcCicLPVjCWbAA9w/132",
                        "my_id": 23,
                        "nick_name": 评论用户的名字,
                        "reply": {
                            "reply_list": [ ]
                        }
                    }
                ],
                "elected_comment_total_cnt": 3, 评论总数
                "enabled": 1,
                "friend_comment": [ ],
                "is_fans": 1,
                "logo_url": "http://wx.qlogo.cn/mmhead/Q3auHgzwzM6GAic0FAHOu9Gtv5lEu5kUqO6y6EjEFjAhuhUNIS7Y2AQ/132",
                "my_comment": [ ],
                "nick_name": 当前用户名,
                "only_fans_can_comment": false
            }
        """
        __biz, _, idx, _ = self.__get_params(article_url)
        getcomment_url = "https://mp.weixin.qq.com/mp/appmsg_comment?action=getcomment&__biz={}&idx={}&comment_id={}&limit=100"
        try:
            url = getcomment_url.format(__biz, idx,
                                        self.__get_comment_id(article_url))
            comment_json = self.s.get(url,
                                      headers=self.headers,
                                      proxies=self.proxies).json()
        except Exception as e:
            print(e)
            comment_json = {}
        return comment_json

    def __get_comment_id(self, article_url):
        """
        获取comment_id
        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        str:
            comment_id获取评论必要参数
        """
        res = self.s.get(article_url, data=self.data, proxies=self.proxies)
        # 使用正则提取comment_id
        comment_id = re.findall(r'comment_id = "\d+"',
                                res.text)[0].split(" ")[-1][1:-1]
        return comment_id

    def __get_params(self, article_url):
        """
        解析文章url, 获取必要的请求参数
        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        (str, str, str, str):
            __biz, mid, idx, sn
        """
        # 简单验证文章的url是否正确
        self.__verify_url(article_url)

        # 切分url, 提取相应的参数
        string_lst = article_url.split("?")[1].split("&")
        dict_value = [string[string.index("=") + 1:] for string in string_lst]
        __biz, mid, idx, sn, *_ = dict_value
        sn = sn[:-3] if sn[-3] == "#" else sn

        return __biz, mid, idx, sn

    def __get_appmsgext(self, article_url):
        """
        获取每篇文章具体信息
        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        json:
            文章具体信息的json
            {
                'advertisement_info': [],
                'advertisement_num': 0,
                'appmsgstat': {'is_login': True,
                'like_num': 12,
                'liked': False,
                'read_num': 288,
                'real_read_num': 0,
                'ret': 0,
                'show': True},
                'base_resp': {'wxtoken': 2045685972},
                'reward_head_imgs': []
            }
        """
        __biz, mid, idx, sn = self.__get_params(article_url)

        # 将params参数换到data中请求。这一步貌似不换也行
        origin_url = "https://mp.weixin.qq.com/mp/getappmsgext?"
        appmsgext_url = origin_url + "appmsg_token={}&x5=0".format(
            self.appmsg_token)
        self.data["__biz"] = __biz
        self.data["mid"] = mid
        self.data["sn"] = sn
        self.data["idx"] = idx

        # appmsgext_url = origin_url + "__biz={}&mid={}&sn={}&idx={}&appmsg_token={}&x5=1".format(
        #     __biz, mid, sn, idx, self.appmsg_token)
        appmsgext_json = requests.post(appmsgext_url,
                                       headers=self.headers,
                                       data=self.data,
                                       proxies=self.proxies).json()

        if "appmsgstat" not in appmsgext_json.keys():
            raise Exception(
                "get info error, please check your cookie and appmsg_token")
        return appmsgext_json



# coding: utf-8
import os
from pprint import pprint
from wechatarticles import ArticlesUrls, tools

if __name__ == "__main__":
    # 模拟登录微信公众号平台，获取微信文章的url
    cookie = "cookie"
    token = "token"
    nickname = "nickname"
    query = "query"

    test = ArticlesUrls(cookie=cookie, token=token)
    # articles_sum = test.articles_nums(nickname)
    artiacle_data = test.articles(nickname, begin="0", count="5")
    # officical_info = test.official_info(nickname)

    # print("articles_sum:", end=" ")
    # print(articles_sum)
    print("artcles_data:")
    pprint(artiacle_data)
    # print("officical_info:")
    # pprint(officical_info)

    tools.save_json("test.json", artiacle_data)



# coding: utf-8

from .ArticlesUrls import ArticlesUrls
from .ArticlesInfo import ArticlesInfo
from .ArticlesAPI import ArticlesAPI
from .Url2Html import Url2Html
from .nickname2biz import nickname2biz
from .GetUrls import PCUrls, MobileUrls



# coding: utf-8
import html
import os
import random
import re
import time
from pprint import pprint
import json
import pandas as pd  # 如果需要保存至excel表格的话
import requests
from wechatarticles import ArticlesInfo, ArticlesUrls
from wechatarticles.GetUrls import PCUrls


def flatten(x):
    return [y for l in x for y in flatten(l)] if type(x) is list else [x]


def transfer_url(url):
    url = html.unescape(html.unescape(url))
    return eval(repr(url).replace('\\', ''))


def verify_url(article_url):
    verify_lst = ["mp.weixin.qq.com", "__biz", "mid", "sn", "idx"]
    for string in verify_lst:
        if string not in article_url:
            return False
    return True


def get_all_urls(urls):
    # 获取所有的url
    url_lst = []
    for item in urls:
        url_lst.append(transfer_url(item['app_msg_ext_info']['content_url']))
        if 'multi_app_msg_item_list' in item['app_msg_ext_info'].keys():
            for ss in item['app_msg_ext_info']['multi_app_msg_item_list']:
                url_lst.append(transfer_url(ss['content_url']))

    return url_lst


def get_all_urls_title_date(urls):
    # 获取所有的[url, title, date]
    url_lst = []

    for item in urls:
        timestamp = item['comm_msg_info']['datetime']
        time_local = time.localtime(timestamp)
        # 转换成日期
        time_temp = time.strftime("%Y-%m-%d", time_local)

        # 文章url
        url_temp = transfer_url(item['app_msg_ext_info']['content_url'])

        # 文章标题
        title_temp = item['app_msg_ext_info']['title']
        url_lst.append([url_temp, title_temp, time_temp])

        if 'multi_app_msg_item_list' in item['app_msg_ext_info'].keys():
            for info in item['app_msg_ext_info']['multi_app_msg_item_list']:

                url_temp = transfer_url(info['content_url'])

                title_temp = info['title']
                url_lst.append([url_temp, title_temp, time_temp])

    return url_lst


def method_one(biz, uin, cookie):

    t = PCUrls(biz=biz, uin=uin, cookie=cookie)
    count = 0
    lst = []
    while True:
        res = t.get_urls(key, offset=count)
        if res == []:
            break
        count += 10
        lst.append(res)

    return lst


def get_info_from_url(url):
    html = requests.get(url).text
    try:
        res = re.findall(r'publish_time =.+\|\|?', html)
        date = res[0].split('=')[1].split('||')[0].strip()
    except:
        date = None

    try:
        res = re.findall(r'nickname .+;?', html)
        offical_name = res[0].split('=')[1][:-1].strip()
    except:
        offical_name = None

    try:
        res = re.findall(r'msg_title = .+;?', html)
        aritlce_name = res[0].split('=')[1][:-1].strip()
    except:
        aritlce_name = None

    return date, offical_name, aritlce_name


def save_xlsx(fj, lst):
    df = pd.DataFrame(
        lst,
        columns=['url', 'title', 'date', 'read_num', 'like_num', 'comments'])
    df.to_excel(fj + '.xlsx', encoding='utf-8')


def get_data(url):
    pass


if __name__ == '__main__':
    # 方法一：使用PCUrls。已在win10下测试
    # 需要抓取公众号的__biz参数
    biz = ''
    # 个人微信号登陆后获取的uin
    uin = ''
    # 个人微信号登陆后获取的cookie
    cookie = ''
    # 个人微信号登陆后获取的key，隔段时间更新
    key = ''

    lst = method_one(biz, uin, cookie)

    # 个人微信号登陆后获取的token
    appmsg_token = ''

    # 获取点赞数、阅读数、评论信息
    test = ArticlesInfo(appmsg_token, cookie)
    """
    data_lst = []
    for i, url in enumerate(url_lst):
        item = test.comments(url)
        temp_lst = [url, item]
        try:
            read_num, like_num = test.read_like_nums(url)
            temp_lst.append(read_num)
            temp_lst.append(like_num)
        except:
            print("第{}个爬取失败，请更新参数".format(i + 1))
            break

        data_lst.append(temp_lst)
    """
    # 存储历史文章信息的json
    # data = []
    data = lst
    fj = '公众号名称'
    item_lst = []
    for i, line in enumerate(data, 0):
        print("index:", i)
        # item = json.loads('{' + line + '}', strict=False)
        item = line
        timestamp = item["comm_msg_info"]["datetime"]
        ymd = time.localtime(timestamp)
        date = '{}-{}-{}'.format(ymd.tm_year, ymd.tm_mon, ymd.tm_mday)

        infos = item['app_msg_ext_info']
        url_title_lst = [[infos['content_url'], infos['title']]]
        if 'multi_app_msg_item_list' in infos.keys():
            url_title_lst += [[info['content_url'], info['title']]
                              for info in infos['multi_app_msg_item_list']]

        for url, title in url_title_lst:
            try:
                if not verify_url(url):
                    continue
                comments = test.comments(url)
                # 获取文章阅读数在看点赞数
                read_num, like_num, old_like_num = test.read_like_nums(url)
                print(read_num, like_num, len(comments))
                item_lst.append(
                    [url, title, date, read_num, like_num, comments])
                time.sleep(random.randint(5, 10))
            except Exception as e:
                print(e)
                flag = 1
                break
            finally:
                save_xlsx(fj, item_lst)

        if flag == 1:
            break

    save_xlsx(fj, item_lst)



# coding: utf-8
import os
import re
import time

import requests


class Url2Html(object):
    """根据微信文章链接下载为本地HTML文件"""
    def __init__(self, img_path=None):
        """
        img_path: 本地存储图片的路径，采用绝对路径的方式引用图片。可不下载图片
        """
        self.data_src_re = re.compile(r'data-src="(.*?)"')
        self.data_croporisrc_re = re.compile(r'data-croporisrc="(.*?)"')
        self.src_re = re.compile(r'src="(.*?)"')
        self.img_path = img_path

    def replace_name(self, title):
        """
        对进行标题替换，确保标题符合windows的命名规则
        title: 文章标题
        """
        rstr = r"[\/\\\:\*\?\"\<\>\|]"  # '/ \ : * ? " < > |'
        title = re.sub(rstr, "", title).replace('|', '').replace('\n', '')
        return title

    def download_img(self, url):
        """
        下载图片
        url: 图片链接
        """
        # 根据链接提取图片名
        name = '{}.{}'.format(
            url.split('/')[-2],
            url.split('/')[3].split('_')[-1])
        imgpath = os.path.join(self.img_path, name)
        # 如果该图片已被下载，可以无需再下载，直接返回路径即可
        if os.path.isfile(imgpath):
            with open(imgpath, 'rb') as f:
                img = f.read()
            return imgpath, img

        response = requests.get(url, proxies=self.proxies)
        img = response.content
        with open(imgpath, 'wb') as f:
            f.write(img)
        imgpath = os.path.basename(self.img_path)
        return os.path.join(imgpath, name), img

    def replace_img(self, html):
        """
        根据提供的html源码找出其中的图片链接，并对其进行替换
        html: 文章源码
        """
        data_croporisrc_lst = self.data_croporisrc_re.findall(html)
        data_src_lst = self.data_src_re.findall(html)
        src_lst = self.src_re.findall(html)

        img_url_lst = data_croporisrc_lst + data_src_lst + src_lst
        img_lst = []
        for img_url in img_url_lst:
            if 'mmbiz.qpic.cn' in img_url:
                # print(img_url)
                data_src, img = self.download_img(img_url)
                img_lst.append([data_src, img])
                # 以绝对路径的方式替换图片
                html = html.replace(img_url,
                                    data_src).replace('data-src=', 'src=')
        return html, img_lst

    def get_title(self, html):
        """
        根据提供的html源码提取文章中的标题
        html: 文章源码
        """
        try:
            title = html.split('activity-name">')[1].split('</h2')[0].strip()
            return title
        except Exception as e:
            print(e)
            return ''

    def article_info(self, html):
        """
        根据提供的html源码提取文章中的公众号和作者
        html: 文章源码
        """
        account = html.split('rich_media_meta rich_media_meta_text">'
                             )[1].split('</span')[0].strip()
        author = html.split('id="js_name">')[1].split('</a')[0].strip()
        return account, author

    def get_timestamp(self, html):
        """
        根据提供的html源码提取文章发表的时间戳
        html: 文章源码
        """
        timestamp = int(html.split('ct = "')[1].split('";')[0].strip())
        return timestamp

    def timestamp2date(self, timestamp):
        """
        时间戳转日期
        timestamp: 时间戳
        """
        ymd = time.localtime(timestamp)
        date = '{}-{}-{}'.format(ymd.tm_year, ymd.tm_mon, ymd.tm_mday)
        return date

    def rename_title(self, title, html):
        # 自动获取文章标题
        if title == None:
            title = self.get_title(html)
        if title == '':
            return ''
        title = self.replace_name(title)

        if self.account == None:
            try:
                account_name = self.article_info(html)[0]
            except:
                account_name = '未分类'
        else:
            account_name = self.account
        try:
            date = self.timestamp2date(self.get_timestamp(html))
        except:
            date = ''
        # try:
        if not os.path.isdir(account_name):
            os.mkdir(account_name)
        # except:
        #     account_name = '未分类'
        #     if not os.path.isdir(account_name):
        #         os.mkdir(account_name)

        title = os.path.join(account_name,
                             '[{}]-{}-{}'.format(account_name, date, title))
        return title

    def run(self, url, mode, proxies={'http': None, 'https': None}, **kwargs):
        """
        启动函数
        url: 微信文章链接
        mode: 运行模式
            1: 返回html源码，不下载图片
            2: 返回html源码，下载图片但不替换图片路径
            3: 返回html源码，下载图片且替换图片路径
            4: 保存html源码，下载图片且替换图片路径
        kwargs:
            account: 公众号名
            title: 文章名
            date: 日期
            proxies: 代理
            img_path: 图片下载路径
        """
        self.proxies = proxies 
        if mode == 1:
            return requests.get(url, proxies=proxies).text
        elif mode in [2, 3, 4]:
            if 'img_path' in kwargs.keys():
                self.img_path = kwargs['img_path']
            else:
                return '{} 请输入保存图片路径!'.format(url)
            if mode == 2:
                return requests.get(url, proxies=proxies).text
            elif mode == 3:
                html = requests.get(url, proxies=proxies).text
                html_img, _ = self.replace_img(html)
                return html_img
            else:
                if 'img_path' in kwargs.keys():
                    self.img_path = kwargs['img_path']
                else:
                    return '{} 请输入保存图片路径!'.format(url)
                if mode == 2:
                    return requests.get(url, proxies=proxies).text
                elif mode == 3:
                    html = requests.get(url, proxies=proxies).text
                    html_img, _ = self.replace_img(html)
                    return html_img
                else:
                    if 'account' in kwargs.keys():
                        self.account = kwargs['account']
                    else:
                        self.account = None
                    if 'title' in kwargs.keys():
                        title = kwargs['title']
                    else:
                        title = None
                    if 'date' in kwargs.keys():
                        date = kwargs['date']
                    else:
                        date = None
                    if 'proxies' in kwargs.keys():
                        proxies = kwargs['proxies']
                    else:
                        proxies = None
                    if self.account and title and date:
                        title = os.path.join(
                            self.account,
                            '[{}]-{}-{}'.format(self.account, date,
                                                self.replace_name(title)))
                        if os.path.isfile('{}.html'.format(title)):
                            return 0
                        html = requests.get(url, proxies=proxies).text
                    else:
                        html = requests.get(url, proxies=proxies).text
                        title = self.rename_title(title, html)

                    html_img, _ = self.replace_img(html)
                    with open('{}.html'.format(title), 'w',
                              encoding='utf-8') as f:
                        f.write(html_img)
                    return '{} success!'.format(url)
        else:
            print("please input correct mode num")
            return 'faied!'


if __name__ == '__main__':

    url_lst = [
        'http://mp.weixin.qq.com/s?__biz=MTc5MTU3NTYyMQ==&mid=2650742058&idx=2&sn=1da6e9ddd1a0281e8c548fb30f8387f0&chksm=5afd0c006d8a851648e91e8cdaebfc2ab61d2518cbba369749b0a29cccb4781c622f966e6b04#rd'
    ]
    uh = Url2Html()
    for url in url_lst:
        s = uh.run(url, mode=4, img_path='D:\\imgs')
        print(s)



# coding: utf-8

import time
import json


# 一些tools，如时间戳转换
def timestamp2date(timestamp):
    """
    时间戳转换为日期
    Parameters
    ----------
    timestamp: int or str
        用户账号

    Returns
    -------
    datetime:
        转换好的日期：年-月-日 时:分:秒
    """
    time_array = time.localtime(int(timestamp))
    datetime = time.strftime("%Y-%m-%d %H:%M:%S", time_array)
    return datetime

def save_mongo(data,
                host=None,
                port=None,
                name=None,
                password="",
                dbname=None,
                collname=None):
    """
    存储数据到mongo
    Parameters
    ----------
    data: list
        需要插入的数据
    host: str
        主机名(默认为本机数据库)
    port: int
        mongo所在主机开放的端口，默认为27017
    username: str
        用户名
    password: str
        用户密码
    dbname: str
        远程连接的数据库名
    collname: str
        需要插入的集合名(collection)
    Returns
    -------
    None
    """
    HOST = "localhost"
    PORT = 27017

    # 检查参数
    host = HOST if host is None else host
    port = PORT if port is None else port

    assert isinstance(host, str)
    assert isinstance(name, str)
    assert isinstance(password, str)
    assert isinstance(dbname, str)        
    assert isinstance(collname, str)

    if not isinstance(port, int):
        raise TypeError("port must be an instance of int")

    from pymongo import MongoClient
    # 连接数据库，一次性插入数据
    client = MongoClient(host, port)
    db_auth = client.admin
    db_auth.authenticate(name, password)
    coll = client[dbname][collname]
    coll.insert_many(data)

def save_json(fname, data):
    """
    保存数据为txt格式
    Parameters
    ----------
    fname: str
        保存为txt文件名
    data: list
        爬取到的数据
    Returns
    -------
    None
    """
    assert isinstance(fname, str)

    if ".json" not in fname:
        raise IOError("fname must be json", fname)
    with open(fname, "a+") as f:
        for item in data:
            f.write(json.dumps(item))
            f.write("\n")



# coding: utf-8

from .ArticlesUrls import ArticlesUrls
from .ArticlesInfo import ArticlesInfo


class ArticlesAPI(object):
    """
    整合ArticlesInfo和ArticlesInfo, 方便调用
    """

    def __init__(self,
                 username=None,
                 password=None,
                 official_cookie=None,
                 token=None,
                 appmsg_token=None,
                 wechat_cookie=None,
                 outfile=None):
        """
        初始化参数
        Parameters
        ----------
        username: str
            用户账号
        password: str
            用户密码
        official_cookie : str
            登录微信公众号平台之后获取的cookie
        token : str
            登录微信公众号平台之后获取的token
        wechat_cookie: str
            点开微信公众号文章抓包工具获取的cookie
        appmsg_token: str
            点开微信公众号文章抓包工具获取的appmsg_token

        Returns
        -------
            None
        """
        # 两种登录方式, 扫码登录和手动输入登录
        if (token != None) and (official_cookie != None):
            self.officical = ArticlesUrls(cookie=official_cookie, token=token)
        elif (username != None) and (password != None):
            self.officical = ArticlesUrls(username=username, password=password)
        else:
            raise SystemError("please check your paramse")

        # 支持两种方式， mitmproxy自动获取参数和手动获取参数
        if (appmsg_token == None) and (wechat_cookie == None) and (outfile !=
                                                                   None):
            from .ReadOutfile import Reader
            reader = Reader()
            reader.contral(outfile)
            self.appmsg_token, self.cookie = reader.request(outfile)
        elif (appmsg_token != None) and (wechat_cookie != None):
            self.appmsg_token, self.cookie = appmsg_token, wechat_cookie
        else:
            raise SystemError("please check your params")

        self.wechat = ArticlesInfo(self.appmsg_token, self.cookie)

    def complete_info(self, nickname, begin=0, count=5):
        """
        获取公众号的抓取的文章文章信息
        Parameters
        ----------
        nickname: str
            公众号名称
        begin: str or int
            起始爬取的页数
        count: str or int
            每次爬取的数量，1-5

        Returns
        -------
        list:
            由每个文章信息构成的数组
            [
              {
                'aid': '2650949647_1',
                'appmsgid': 2650949647,
                'comments': 文章评论信息
                    {
                        "base_resp": {
                            "errmsg": "ok", 
                            "ret": 0
                        }, 
                        "elected_comment": [
                            {
                                "content": 用户评论文字, 
                                "content_id": "6846263421277569047", 
                                "create_time": 1520098511, 
                                "id": 3, 
                                "is_from_friend": 0, 
                                "is_from_me": 0, 
                                "is_top": 0, 是否被置顶
                                "like_id": 10001, 
                                "like_num": 3, 
                                "like_status": 0, 
                                "logo_url": "http://wx.qlogo.cn/mmhead/OibRNdtlJdkFLMHYLMR92Lvq0PicDpJpbnaicP3Z6kVcCicLPVjCWbAA9w/132", 
                                "my_id": 23, 
                                "nick_name": 评论用户的名字, 
                                "reply": {
                                    "reply_list": [ ]
                                }
                            }
                        ], 
                        "elected_comment_total_cnt": 3, 评论总数
                        "enabled": 1, 
                        "friend_comment": [ ], 
                        "is_fans": 1, 
                        "logo_url": "http://wx.qlogo.cn/mmhead/Q3auHgzwzM6GAic0FAHOu9Gtv5lEu5kUqO6y6EjEFjAhuhUNIS7Y2AQ/132", 
                        "my_comment": [ ], 
                        "nick_name": 当前用户名, 
                        "only_fans_can_comment": false
                    }, 
                'cover': 封面的url'digest': 文章摘要,
                'itemidx': 1,
                'like_num': 18, 文章点赞数
                'link': 文章的url,
                'read_num': 610, 文章阅读数
                'title': 文章标题,
                'update_time': 更新文章的时间戳
              },
            ]
        如果list为空则说明没有相关文章
        """
        # 获取文章数据
        artiacle_data = self.officical.articles(
            nickname, begin=str(begin), count=str(count))

        # 提取每个文章的url，获取文章的点赞、阅读、评论信息，并加入到原来的json中
        for data in artiacle_data:
            article_url = data["link"]
            comments = self.wechat.comments(article_url)
            read_like_nums = self.wechat.read_like_nums(article_url)
            data["comments"] = comments
            data["read_num"], data["like_num"], data['old_like_num'] = read_like_nums

        return artiacle_data

    def __extract_info(self, articles_data):
        # 提取每个文章的url，获取文章的点赞、阅读、评论信息，并加入到原来的json中
        for data in articles_data:
            article_url = data["link"]
            comments = self.wechat.comments(article_url)
            read_like_nums = self.wechat.read_like_nums(article_url)
            data["comments"] = comments
            data["read_num"], data["like_num"], data['old_like_num'] = read_like_nums

        return articles_data

    def continue_info(self, nickname, begin=0):
        """
        自动获取公众号的抓取的文章文章信息，直到爬取失败为止
        Parameters
        ----------
        nickname: str
            公众号名称
        begin: str or int
            起始爬取的页数

        Returns
        -------
        list:
            由每个文章信息构成的数组
            [
              {
                'aid': '2650949647_1',
                'appmsgid': 2650949647,
                'comments': 文章评论信息
                    {
                        "base_resp": {
                            "errmsg": "ok", 
                            "ret": 0
                        }, 
                        "elected_comment": [
                            {
                                "content": 用户评论文字, 
                                "content_id": "6846263421277569047", 
                                "create_time": 1520098511, 
                                "id": 3, 
                                "is_from_friend": 0, 
                                "is_from_me": 0, 
                                "is_top": 0, 是否被置顶
                                "like_id": 10001, 
                                "like_num": 3, 
                                "like_status": 0, 
                                "logo_url": "http://wx.qlogo.cn/mmhead/OibRNdtlJdkFLMHYLMR92Lvq0PicDpJpbnaicP3Z6kVcCicLPVjCWbAA9w/132", 
                                "my_id": 23, 
                                "nick_name": 评论用户的名字, 
                                "reply": {
                                    "reply_list": [ ]
                                }
                            }
                        ], 
                        "elected_comment_total_cnt": 3, 评论总数
                        "enabled": 1, 
                        "friend_comment": [ ], 
                        "is_fans": 1, 
                        "logo_url": "http://wx.qlogo.cn/mmhead/Q3auHgzwzM6GAic0FAHOu9Gtv5lEu5kUqO6y6EjEFjAhuhUNIS7Y2AQ/132", 
                        "my_comment": [ ], 
                        "nick_name": 当前用户名, 
                        "only_fans_can_comment": false
                    }, 
                'cover': 封面的url'digest': 文章摘要,
                'itemidx': 1,
                'like_num': 18, 文章点赞数
                'link': 文章的url,
                'read_num': 610, 文章阅读数
                'title': 文章标题,
                'update_time': 更新文章的时间戳
              },
            ]
        如果list为空则说明没有相关文章
        """
        artiacle_datas = []
        count = 5
        while True:
            try:
                # 获取文章数据
                artiacle_datas.append(
                    self.officical.articles(
                        nickname, begin=str(begin), count=str(count)))
            except Exception as e:
                print(e)
                break
            begin += count
            if begin > 40:
                break

        def flatten(x):
            return [y for l in x for y in flatten(l)] if type(x) is list else [x]
        # flatten = lambda x: [y for l in x for y in flatten(l)] if type(x) is list else [x]
        print("第{}篇文章爬取失败，请过段时间再次尝试或换个帐号继续爬取".format(begin))
        return self.__extract_info(flatten(artiacle_datas))



# coding: utf-8
import requests


class PCUrls(object):
    """
    通过PC端的wechat，获取需要爬取的微信公众号的推文链接
    """
    def __init__(self,
                 biz,
                 uin,
                 cookie,
                 proxies={
                     'http': None,
                     'https': None
                 }):
        """
        初始化参数
        Parameters
        ----------
        __biz: str
            需要爬取公众号的id
        uin: str
            用户id
        cookies : str
            登录微信后获取的cookie
        Returns
        -------
            None
        """
        self.s = requests.session()
        self.__biz = biz
        self.uin = uin
        self.headers = {'Cookies': cookie}
        self.proxies = proxies

    def get_urls(self, key, offset='0'):
        """
        获取urls
        Parameters
        ----------
        key: str
            个人微信号登陆后获取的key
        offset: str or int
            获取起始的页数，从0开始，每次递增10（可以大于10，但是不好确认参数，所以递增10，之后再去重）
        Returns
        ----------
        list:
        由每个文章信息构成的数组，主要获取的参数item['app_msg_ext_info']['content_url'], item['app_msg_ext_info']['title'], item['comm_msg_info']['datetime']
        import html
        消除转义 html.unescape(html.unescape(url)); eval(repr(url).replace('\\', ''))
        [
            {
                'app_msg_ext_info': {
                    'audio_fileid': 0,
                    'author': '',
                    'content': '',
                    'content_url': 文章url，存在转义符'/'需要去除,
                    'copyright_stat': 100,
                    'cover': 文章封面url，存在转义符'/'需要去除,
                    'del_flag': 1,
                    'digest': '',
                    'duration': 0,
                    'fileid': 0,
                    'is_multi': 0,
                    'item_show_type': 8,
                    'malicious_content_type': 0,
                    'malicious_title_reason_id': 0,
                    'multi_app_msg_item_list': [],
                    'play_url': '',
                    'source_url': '',
                    'subtype': 9,
                    'title': 文章标题
                },
                'comm_msg_info': {
                    'content': '',
                    'datetime': 1536930840,
                    'fakeid': '2394588245',
                    'id': 1000000262,
                    'status': 2,
                    'type': 49
                }
            }
        ]
        """
        self.params = {
            'action': 'getmsg',
            '__biz': self.__biz,
            'f': 'json',
            'offset': str(offset),
            'count': '10',
            'uin': self.uin,
            'key': key,
        }
        origin_url = 'https://mp.weixin.qq.com/mp/profile_ext'

        msg_json = self.s.get(origin_url,
                              params=self.params,
                              headers=self.headers,
                              proxies=self.proxies).json()
        if 'general_msg_list' in msg_json.keys():
            lst = [
                item for item in eval(msg_json['general_msg_list'])['list']
                if 'app_msg_ext_info' in item.keys()
            ]
            return lst

        raise Exception(
            'Failure:\n1.params is error, please check your params\n2.key is lose efficacy, please update your key'
        )


class MobileUrls(object):
    """
    通过移动端的wechat，获取需要爬取的微信公众号的推文链接
    """
    def __init__(self, biz, cookie):
        """
        初始化参数
        Parameters
        ----------
        __biz: str
            需要爬取公众号的id
        cookie : str
            登录微信后获取的cookie
        Returns
        -------
            None
        """
        self.s = requests.session()
        self.__biz = biz
        self.headers = {
            'User-Agent':
            'Mozilla/5.0 AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0Chrome/57.0.2987.132 MQQBrowser/6.2 Mobile',
            'Cookie': cookie
        }

    def get_urls(self, appmsg_token, offset='0'):
        """
        获取urls
        Parameters
        ----------
        appmsg_token: str
            个人微信号登陆后获取的token
        offset: str or int
            获取起始的页数，从0开始，每次递增10（可以大于10，但是不好确认参数，所以递增10，之后再去重）
        Returns
        ----------
        list:
        由每个文章信息构成的数组，主要获取的参数item['app_msg_ext_info']['content_url'], item['app_msg_ext_info']['title'], item['comm_msg_info']['datetime']
        import html
        消除转义 html.unescape(html.unescape(url)); eval(repr(url).replace('\\', ''))
        [
            {
                'app_msg_ext_info': {
                    'audio_fileid': 0,
                    'author': '',
                    'content': '',
                    'content_url': 文章url，存在转义符'/'需要去除,
                    'copyright_stat': 100,
                    'cover': 文章封面url，存在转义符'/'需要去除,
                    'del_flag': 1,
                    'digest': '',
                    'duration': 0,
                    'fileid': 0,
                    'is_multi': 0,
                    'item_show_type': 8,
                    'malicious_content_type': 0,
                    'malicious_title_reason_id': 0,
                    'multi_app_msg_item_list': [],
                    'play_url': '',
                    'source_url': '',
                    'subtype': 9,
                    'title': 文章标题
                },
                'comm_msg_info': {
                    'content': '',
                    'datetime': 1536930840,
                    'fakeid': '2394588245',
                    'id': 1000000262,
                    'status': 2,
                    'type': 49
                }
            }
        ]
        """
        self.params = {
            'action': 'getmsg',
            '__biz': self.__biz,
            'f': 'json',
            'offset': str(offset),
            'count': '10',
            'appmsg_token': appmsg_token,
        }
        origin_url = 'https://mp.weixin.qq.com/mp/profile_ext'

        msg_json = self.s.get(origin_url,
                              params=self.params,
                              headers=self.headers,
                              proxies=self.proxies).json()
        if 'general_msg_list' in msg_json.keys():
            lst = [
                item for item in eval(msg_json['general_msg_list'])['list']
                if 'app_msg_ext_info' in item.keys()
            ]
            return lst

        raise Exception(
            'Failure:\n1.params is error, please check your params\n2.key is lose efficacy, please update your key'
        )



# coding: utf-8
import os
import re
import time

import requests
from bs4 import BeautifulSoup as bs

from .ArticlesUrls import ArticlesUrls


class nickname2biz(object):
    """输入公众号名称转biz"""
    def __init__(self,
                 cookie,
                 token=None,
                 method=None,
                 t=120,
                 proxies={
                     'http': None,
                     'https': None,
                 }):
        """
        cookie: 平台登录的cookie
        token: 官方获取时需要token
        method: 三种获取方式，三选一
        官方、清博、西瓜
        ['xigu', 'qingbo', 'office']
        实测西瓜一次性可获取较多

        t: 获取一次时间间隔
        西瓜10s
        清博10s
        公众号120s
        """
        self.headers = {
            "User-Agent":
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.181 Safari/537.36",
            "Cookie": cookie
        }
        assert method in ['xigua', 'qingbo', 'office']
        self.method = method
        self.cookie = cookie
        self.token = token
        self.t = t
        self.biz_name = '{}, {}'
        self.proxies = proxies

    def run(self, nickname_lst):
        if self.method == 'xigua':
            return self.xigua(nickname_lst)
        elif self.method == 'qingbo':
            return self.qingbo(nickname_lst)
        else:
            s = ArticlesUrls(cookie=self.cookie,
                             token=self.token,
                             proxies=self.proxies)
            return self.office(s, nickname_lst)

    def office(self, s, nickname_lst):
        self.res_lst = []
        for nickname in nickname_lst:
            try:
                officical_infos = s.official_info(nickname)
                if officical_infos:
                    officical_info = officical_infos[0]
                    biz = officical_info['fakeid']
                    tmp = self.biz_name.format(biz, officical_info['nickname'])
                    self.res_lst.append(tmp)
                time.sleep(self.t)
            except Exception as e:
                print(e)
                return self.res_lst
        return self.res_lst

    def xigua(self, nickname_lst):
        url = 'https://data.xiguaji.com/Search/SearchAct/?type=1&key={}'
        self.res_lst = []
        for nickname in nickname_lst:
            try:
                s = requests.get(url.format(nickname),
                                 headers=self.headers,
                                 proxies=self.proxies)
                soup = bs(s.text, 'lxml')
                infos = soup.find_all(class_="number-details")
                if infos:
                    info = infos[0]
                    nickname = info.h3.text.strip()
                    try:
                        biz = info.img['src'].split('__biz=')[1].split('&')[0]
                    except Exception as e:
                        continue
                        print(e)
                    self.res_lst.append(self.biz_name.format(biz, nickname))
                time.sleep(self.t)
            except Exception as e:
                print(e)
                return self.res_lst
        return self.res_lst

    def qingbo(self, nickname_lst):
        url = 'http://www.gsdata.cn/query/wx?q={}'
        self.res_lst = []
        for nickname in nickname_lst:
            try:
                s = requests.get(url.format(nickname),
                                 headers=self.headers,
                                 proxies=self.proxies)
                biz_lst = re.findall(
                    r'<input type="hidden" class="biz" value="(.+)">?', s.text)
                if biz_lst != []:
                    nicknames_lst = re.findall(
                        r'<span class="color-pink">(.+)</span>', s.text)
                    tmp = self.biz_name.format(biz_lst[0], nicknames_lst[0])
                    self.res_lst.append(tmp)
                time.sleep(self.t)
            except Exception as e:
                print(e)
                return self.res_lst
        return self.res_lst


if __name__ == '__main__':
    nickname_lst = ['科技美学', 'AppSo', 'InfoQ']
    cookie = ''
    nb = nickname2biz(cookie, method='xigua', t=10)
    res_lst = nb.run(nickname_lst)

    fname = '1.txt'
    with open(fname, 'w', encoding='utf-8') as f:
        f.write('\n'.join(item for item in res_lst))



# coding: utf-8
import os
from pprint import pprint
from wechatarticles.ReadOutfile import Reader

if __name__ == '__main__':
    outfile = 'outfile'
    reader = Reader()
    reader.contral(outfile)
    appmsg_token, cookie = reader.request(outfile)
    print(appmsg_token, cookie)
